<?php

namespace App\Http\Controllers;

use App\Models\User;
use App\Traits\GlobalTrait;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;

class Base extends Controller
{

    use GlobalTrait;
    // !==================
    // ! For Json Response
    // !==================
    public static function success($msg = 'Task successful!!', $data = [], $type = null)
    {

        $msg = GlobalTrait::translate($msg);

        return response()->json([
            'success' => true,
            'code' => 200,
            'message' => $msg,
            'data' => $data,
            'type' => $type
        ]);
    }
    public static function error($msg = 'An error occurred!!', $error = [], $type = null)
    {
        $msg = GlobalTrait::translate($msg);
        return response()->json([
            'success' => false,
            'code' => 400,
            'message' => $msg,
            'data' => $error,
            'type' => $type
        ]);
    }
    public static function validation($validator)
    {
        // if ($validator->fails()) return Base::error($validator->errors()->first(), $validator->errors());
        return response()->json([
            'success' => false,
            'code' => 400,
            'message' => $validator->errors()->first(),
            'data' => $validator->errors(),
            'type' => 'validation'
        ]);
    }
    public static function exception($e)
    {
        // return Base::error($e->getMessage()." ".$e->getFile()." ".$e->getLine());

        $exception_msg = $e->getMessage() . " " . $e->getFile() . " " . $e->getLine();

        error_log("⚠️⚠️⚠️⚠️⚠️⚠️: " . $exception_msg);

        // $msg = 'An error occurred!!'; // ! For Production
        $msg = $exception_msg; // ! For Development
        // $msg = GlobalTrait::translate($msg); //! translated message

        return response()->json([
            'success' => false,
            'code' => 400,
            'message' => $msg,
            'data' => $e,
            'type' => 'exception'
        ]);
    }



    // !======================
    // ! For Internal Checking
    // !======================
    public static function pass($msg = 'Task successful!!', $data = [])
    {
        // $msg = GlobalTrait::translate($msg);
        return (object) collect([
            'success' => true,
            'message' => $msg,
            'data' => $data
        ])->all();
    }

    public static function fail($msg = 'An error occurred!!', $error = [], $err_type = null)
    {
        // $msg = GlobalTrait::translate($msg);
        return (object) collect([
            'success' => false,
            'message' => $msg,
            'data' => $error
        ])->all();
    }

    public static function exception_fail($e)
    {
        $exception_msg = $e->getMessage() . " " . $e->getFile() . " " . $e->getLine();

        error_log("⚠️⚠️⚠️⚠️⚠️⚠️: " . $exception_msg);

        $msg = $exception_msg; // ! For Development
        // $msg = GlobalTrait::translate($msg); //! for translation

        return (object) collect([
            'success' => false,
            'code' => 400,
            'message' => $msg,
            'data' => $e,
            'type' => 'exception'
        ])->all();
    }


    public static function now()
    {
        return Carbon::now();
    }

    public static function carbonDateParse($date)
    {
        return Carbon::parse($date)->isoFormat('D MMM, YYYY');
    }

    public static function timeParse($time)
    {
        return Carbon::parse($time)->format('H:i');
    }

    public static function carbonParse($date)
    {
        return Carbon::parse($date);
    }

    public static function dateTimeConcat($date, $time)
    {

        return Carbon::parse($date . ' ' . $time);
    }

    public static function AuthUser(Request $request)
    {
        $auth_user = Auth::user();

        if ($auth_user->role == 'sa' || $auth_user->role == 'admin') {
            $user = User::find($request->user_id);
        } else $user = $auth_user;

        return $user;
    }

    public static function isAdmin()
    {
        $auth_user = Auth::user();

        if ($auth_user->role == 'sa' || $auth_user->role == 'admin') {
            return true;
        } else return false;
    }

    public static function getUserType()
    {
        $auth_user = Auth::user();

        if ($auth_user->role == 'sa' || $auth_user->role == 'admin') {
            return 'admin';
        } else return $auth_user->role;
    }

    //! tax calculation, send amount and tex percentage to get tax amount as integer value

    public static function taxCalculation($amount, $tax_percentage)
    {
        // $tax_amount = 8.59999999;
        $tax_amount = ($amount * $tax_percentage) / 100;
        return round($tax_amount);
    }




    // !===============
    // ! Basic Template
    // !===============
    public function template(Request $request)
    {
        try {
            $validator =  Validator::make($request->all(), [
                'email' => ['required', 'email', 'max:255'],
                'type' => 'required|in:school,instructor',
                'days' => 'required|array|distinct|min:1|in:mon,tue,wed,thu,fri,sat,sun',
                "days.*"  => "required|distinct|min:3",
            ]);
            if ($validator->fails()) return Base::validation($validator);
            return Base::success('');
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }
}
