<?php

namespace App\Http\Controllers;

use App\Http\Resources\UserResource;
use App\Models\Notification;
use App\Models\Setting;
use App\Services\Interface\CategoryInterface;
use App\Services\Interface\PostCardInterface;
use App\Services\Interface\ShopBoostedInterface;
use App\Services\Repositories\CategoryRepository;
use App\Services\Repositories\OrderRepository;
use App\Services\Repositories\ShopRepository;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class HomeController extends Controller
{
    public function index(OrderRepository $i_orders, CategoryRepository $i_category, PostCardInterface $i_post_card, ShopRepository $shops, ShopBoostedInterface $shop_boost, Request $request)
    {
        try {
            // $shops = $shops->appHome2(6);
            $popular_cards_data = $i_post_card->popularIndex(6);
            $category = $i_category->getActiveCategory();
            
            // $shops = $shops->success ? $shops->data : [];
            $shops = [];

            $popular_cards = $popular_cards_data->success ? $popular_cards_data->data : [];


            $recommended_cards_data = $i_post_card->recommendedIndex(6);
            $boosted_shop = $shop_boost->boostedShop(1);
            $user = null;

            $user = Auth::guard('api')->user();


            $unseen_count = null;

            if ($user != null) {
                $orders_filter = [
                    'customer_id' => $user->id,
                    // 'status' => 'pending'
                    'take' => 6
                ];
                $pending_orders = $i_orders->homePendingOrder($orders_filter);
                $pending_orders = $pending_orders->success ? $pending_orders->data : [];

                $unseen_count = Notification::where('to_id', $user->id)->where('is_seen', false)->get()->count();
            } else $pending_orders = [];



            $recommended = [];



            $recommended = $recommended_cards_data->success ? $recommended_cards_data->data : [];
            $boosted_shop = $boosted_shop->success ? $boosted_shop->data : null;
            $category = $category->success ? $category->data : [];

            $ship_fee = Setting::where('type', 'ship_fee')->first(['type', 'value']);
            $auth_fee = Setting::where('type', 'auth_fee')->first(['type', 'value']);
            $tax_percentage = Setting::where('type', 'tax_percentage')->first(['type', 'value']);

            $auth_fee = $auth_fee->value ?? 0;



            $general_info = [
                'shipping_fee' => $ship_fee->value ?? 0,
                'auth_fee' => $auth_fee,
                'authority_fee' => $auth_fee + Base::taxCalculation($auth_fee, $tax_percentage->value ?? 10) ?? 0,
                'tax_percentage' => $tax_percentage->value ?? 0,
                'new_notification_count' => $unseen_count ?? 0,
            ];


            if ($request->header('app_type') == 'app' || $request->header('app-type') == 'app') {

                $data = [
                    'shops' => $shops,
                    'popular_cards' => $popular_cards,
                    'boosted_shop' => $boosted_shop,
                    'recommended_cards' => $recommended,
                    'general_info' => $general_info,
                    'category' => $category,

                    'pending_orders' => $pending_orders,
                    'user' => $user ? new UserResource($user) : null,

                ];
            } else {
                $testimonials = [
                    [
                        "name" => "Maria Sharpovba",
                        "text" => "Curabitur aliquet quam id dui posuere blandit. Vivamus magna justo, lacinia eget consectetur sed, convallis at tellus.",
                        "rate" => 5,
                    ],
                    [
                        "name" => "John Doe",
                        "text" => "Curabitur aliquet quam id dui posuere blandit. Vivamus magna justo, lacinia eget consectetur sed, convallis at tellus.",
                        "rate" => 4,
                    ],
                    [
                        "name" => "Marial Hill",
                        "text" => "Curabitur aliquet quam id dui posuere blandit. Vivamus magna justo, lacinia eget consectetur sed, convallis at tellus.",
                        "rate" => 5,
                    ],
                    [
                        "name" => "Dr. Strange",
                        "text" => "Curabitur aliquet quam id dui posuere blandit. Vivamus magna justo, lacinia eget consectetur sed, convallis at tellus.",
                        "rate" => 5,
                    ],
                ]; // TODO: Make Dynamic

                $data = [
                    'shops' => $shops,
                    'popular_cards' => $popular_cards,
                    'category' => $category,
                    'testimonials' => $testimonials,
                    'general_info' => $general_info,
                ];
            }

            $popular_shops = $i_post_card->popularIndex(5);
            if (!$popular_shops->success) return Base::error($popular_shops->message);


            return Base::success('Home data fetch successfully!', $data);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function getCategory(CategoryInterface $i_category)
    {
        try {
            $data = $i_category->getActiveCategory();

            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }
}
