<?php

namespace App\Http\Controllers\Shop;

use App\Http\Controllers\Base;
use App\Http\Controllers\Controller;
use App\Http\Requests\OrderAction;
use App\Services\Interface\OrderInterface;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class OrderController extends Controller
{
    //
    private OrderInterface $orderRepository;

    public function __construct(OrderInterface $orderRepository)
    {
        $this->orderRepository =$orderRepository;
    }

    public function index(Request $request)
    {
        try{
            $data = $this->orderRepository->shopOrderIndex();
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);

        }
        catch(Exception $e)
        {
            return Base::exception($e);
        }
    }

    public function orderShow(OrderInterface $i_order, Request $request)
    {
        try {
            $validator =  Validator::make($request->all(), [
                'id' => 'required',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $res = $i_order->orderHistoryShow($request->id);
            if (!$res->success) return Base::error($res->message);

            return Base::success('Order Details found', $res->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function orderAction(OrderAction $request){
        try{
            $data = $this->orderRepository->orderAction($request);
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        }
        catch(Exception $e){
            return Base::error($e);
        }
    }

}
