<?php

namespace App\Http\Controllers\Shop;

use App\Http\Controllers\Base;
use App\Http\Controllers\Controller;
use App\Models\Order;
use App\Services\Interface\OrderInterface;
use App\Services\Interface\PostCardInterface;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class ShopController extends Controller
{
    //
    public function getDashboard(OrderInterface $orderInterface, PostCardInterface $i_card)
    {
        try {
            $user = Auth::user();
            $recent_orders = $this->getRecentOrders($user);
            $total_earnings = $this->getTotalEarning($user);
            $analytics = $this->getAnalytics($user);

            $top_selling_cards_of_month = $this->getTopSellingCardsOfMonth($user);

            $shop_cards = $this->shopCards($i_card, $user);
            $shop_cards = $shop_cards->success ?  $shop_cards->data : null;
            $shop_cards = json_encode($shop_cards);
            $shop_cards = json_Decode($shop_cards);


            $pending_order = $this->pendingOrder($orderInterface);

            $pending_orders = $pending_order->success ?  $pending_order->data : null;
            $pending_orders = json_encode($pending_orders);
            $pending_orders = json_Decode($pending_orders);

            $completeOrder = $this->completeOrder($orderInterface);
            $completeOrder = $completeOrder->success ?  $completeOrder->data : null;
            $completeOrder = json_encode($completeOrder);
            $completeOrder = json_Decode($completeOrder);

            // return $completeOrder->total;
            $data = [
                'shop_cards' => $shop_cards->total,
                'complete_order' => $completeOrder->total,
                'pending_orders' => $pending_orders->total,
                'total_earnings' => $total_earnings,
                'recent_orders' => $recent_orders,
                'top_selling_cards_of_month' => $top_selling_cards_of_month,
                'analytics' => $analytics,
            ];
            return Base::success('Dashboard Information', $data); // DONE
        } catch (Exception $e) {
            return Base::error($e->getMessage());
        }
    }

    public function shopCards($i_card, $user)
    {
        $shop_id = $user->id;
        return    $data = $i_card->filter($shop_id, null);
        // return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
    }

    public function pendingOrder($orderInterface)
    {
        $request = [
            'status' => 'ongoing'
        ];
        return $data = $orderInterface->orderHistoryFilter($request);
        // return $data->success ? Base::pass($data->message, $data->data) : Base::fail($data->message);
    }

    public function completeOrder($orderInterface)
    {
        $request = [
            // 'take' => 10000,
            'status' => ['completed']
        ];
        return $data = $orderInterface->orderHistoryFilter($request);
        // return $data->success ? Base::pass($data->message, $data->data) : Base::fail($data->message);
    }

    public function getTotalEarning($user){
        $shop_id = $user->id;
        $orders = Order::with('postCard')
            ->whereHas('postCard', function ($q) use ($shop_id) {
                $q->where('shop_id', $shop_id);
            })->get();
        $total_price = 0;
        foreach ($orders as $item) {
            $total_price = $total_price + ($item->price * $item->quantity);
        }

        return $total_price;
    }

    private function getTopSellingCardsOfMonth($user)
    {

        //! TODO: Need to refactor this function, this is temporary solution
        $data = [];
        $card = [];

        $month = Carbon::now()->month;
        $shop_id = $user->id;
        $orders = Order::whereMonth('created_at', $month)->with('postCard')
            ->whereHas('postCard', function ($q) use ($shop_id) {
                $q->where('shop_id', $shop_id);
            })->get(['card_id']);
        $item2 = [];

        foreach ($orders as $item) {
            $order_count = Order::whereMonth('created_at', $month)->where('card_id', $item->card_id)->where('status', 'completed')->count();
            $order_total = Order::whereMonth('created_at', $month)->where('card_id', $item->card_id)->where('status', 'completed')->get();
            $total_price = 0;
            foreach ($order_total as $k) {
                $total_price = $total_price + $k->order_total;
            }
            $item2 = [
                "card_id" => $item->card_id,
                "card_name" => $item->card_name,
                "card_image" => $item->card_image,
                "card_price" => $item->card->price,
                "card_total_order" => $order_count,
                "card_total_sale" => $total_price,
            ];
            array_push($data, $item2);
        }

        $data = collect($data)->unique('card_id')->sortByDesc('card_total_sale')->take(5)->values()->all();

        return $data;
    }

    private function getRecentOrders($user)
    {
        $shop_id = $user->id;
        $data = [];
        $orders = Order::with('postCard')
            ->whereHas('postCard', function ($q) use ($shop_id) {
                $q->where('shop_id', $shop_id);
            })->orderBy('created_at', 'desc')->take(5)->get();

        foreach ($orders as $order) {
            $required_order_data = [
                'id' => $order->id,
                'card_name' => $order->postCard->name,
                'card_image' => $order->postCard->image ?? '',
                'quantity' => $order->quantity,
                'price' => $order->price,
                'payment_method' => $order->payment_method,
                'customer_name' => $order->user->name,
                'customer_phone' => $order->user->phone ?? '',
                'address' => $order->ship_address,
                'code' => $order->code,
                'status' => $order->status,
            ];
            array_push($data, $required_order_data);
        }

        return $data;
    }

    private function getAnalytics($user)
    {
        $data = [];
        $month = Carbon::now()->month;

        $shop_id = $user->id;
        $total_sale_order = Order::with('postCard')
            ->whereHas('postCard', function ($q) use ($shop_id) {
                $q->where('shop_id', $shop_id);
            })->whereMonth('created_at', $month)->where('status', 'completed')->count();
        $total_pending_order = Order::with('postCard')
            ->whereHas('postCard', function ($q) use ($shop_id) {
                $q->where('shop_id', $shop_id);
            })->whereMonth('created_at', $month)->whereIn('status', ['pending', 'processing'])->count();
        $total_cancel_order = Order::with('postCard')
            ->whereHas('postCard', function ($q) use ($shop_id) {
                $q->where('shop_id', $shop_id);
            })->whereMonth('created_at', $month)->whereIn('status', ['canceled', 'rejected'])->count();

            if($total_sale_order != 0 || $total_pending_order != 0 || $total_cancel_order != 0) {
                $s_total_order = (100 / ($total_sale_order + $total_pending_order + $total_cancel_order)) * $total_sale_order;
                $p_total_order = (100 / ($total_sale_order + $total_pending_order + $total_cancel_order)) * $total_pending_order;
                $c_total_order = (100 / ($total_sale_order + $total_pending_order + $total_cancel_order)) * $total_cancel_order;
            }else{
                $s_total_order = $total_sale_order;
                $p_total_order = $total_pending_order;
                $c_total_order = $total_cancel_order;
            }

        return $data = [
            "total_sale_order" => ($s_total_order),
            "total_pending_order" => ($p_total_order),
            "total_cancel_order" => ($c_total_order),
        ];
    }
}
