<?php

namespace App\Http\Requests;

use App\Http\Controllers\Base;
use Illuminate\Contracts\Validation\Validator;
use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Http\Exceptions\HttpResponseException;

class AddVoucher extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, mixed>
     */
    public function rules()
    {
        return [
            "type" => "required|in:global,customer,shop",
            "start_date" => "required",
            "end_date" => "required",
            "amount" => "required|integer",
            "limit" => "required|integer",
            "voucher_code" => "string|max:30|unique:vouchers,voucher_code",

        ];
    }
    public function failedValidation(Validator $validator)
    {
        throw new HttpResponseException(Base::validation($validator));
        // return Base::validation($validator);
    }

    public function messages()
    {
        return [
            "type.required" => "Voucher type is required",
            "type.in" => "Select proper type.",
            "start_date.required" => "Start Date is required.",
            "end_date.required" => "End Date is required.",
            "amount.required" => "Amount is required.",
            "voucher_code.string" => "Voucher Code must be string.",
            "voucher_code.max" => "Voucher code max length is 30.",
            "voucher_code.unique" => "Voucher code should be unique.",
        ];
    }
}
