<?php

namespace App\Models;

use App\Http\Controllers\Base;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Route;

class Order extends Model
{
    use HasFactory;
    use SoftDeletes;
    protected $appends = ['order_date', 'card_name', 'card_image', 'order_total', 'shop_image_base64'];
    protected $casts = [
        'id' => 'integer',
        'price' => 'double',
        'quantity' => 'integer',
        'authority_fee' => 'integer',
        'shipping_fee' => 'integer',
        'discount' => 'integer',
        'uer_id' => 'integer',
    ];

    public function postCard()
    {
        return $this->hasOne(PostCard::class, 'id', 'card_id');
    }
    public function card()
    {
        return $this->hasOne(PostCard::class, 'id', 'card_id');
    }
    public function postCardWithShop()
    {
        return $this->hasOne(PostCard::class, 'id', 'card_id')->with('shop');
    }

    public function orderPayment()
    {
        return $this->hasOne(OrderPayment::class, 'order_id')->latest();
    }

    public function getOrderDateAttribute()
    {
        return Base::carbonParse($this->created_at);
    }

    public function getCardNameAttribute()
    {
        return $this->postCard->name;
    }

    public function getCardImageAttribute()
    {
        return $this->postCard->image;
    }

    public function getShopImageBase64Attribute()
    {
        $image_url = $this->postCard->shop->image_url ?? null;


        $base64 = null;

        $auth_user = Auth::guard('api')->user();

        if ($auth_user != null){
            if ($auth_user->role == 'sa' || $auth_user->role == 'admin') {
                if(Route::is('admin.order.show')){
                    if ($image_url != null) {
                        $path = $image_url;
                        $type = pathinfo($path, PATHINFO_EXTENSION);
                        $data = (file_exists($path)) ? file_get_contents($path) : 0;;
                        $base64 = 'data:image/' . $type . ';base64,' . base64_encode($data);
                    }
                }
                // $order->text_image_base64 = $base64;
            }
        }



        return $base64;
    }

    public function getOrderTotalAttribute()
    {
        // $shipping_fee = $this->shipping_fee ?? 0;

        $this->unsetRelation('postCard');
        $card_price = ($this->price * $this->quantity)  - $this->discount;
        $card_price = $card_price < 0 ? 0 : $card_price;
        $total = ( $card_price + $this->price_tax + $this->shipping_fee + $this->authority_fee + $this->authority_fee_tax) ;
        return $total;
    }

    public function user()
    {
        return $this->belongsTo(User::class, 'user_id');
    }
    public function customer()
    {
        return $this->belongsTo(User::class, 'user_id');
    }
}
