<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Auth;

class PostCard extends Model
{
    use HasFactory;
    protected $hidden = ['sold'];
    protected $appends = ['sold_count', 'is_favorite'];

    protected $casts = [
        'price' => 'double',
        'rate' => 'integer',
        'is_active' => 'integer',
    ];

    public function shop()
    {
        return $this->belongsTo(User::class, 'shop_id','id');
    }
    
    public function category()
    {
        return $this->belongsTo(Category::class);
    }
    public function scopeActiveCategory($query)
    {
        return $query->whereHas('category', function ($q) {
            $q->where('is_active', true);
        });
    }

    public function orders()
    {
        return $this->hasMany(Order::class, 'card_id');
    }
    public function pendingOrders()
    {
        return $this->hasMany(Order::class, 'card_id')->whereIn('status', ['pending', 'processing']);
    }
    public function sold()
    {
        return $this->hasMany(Order::class, 'card_id')->where('status', 'completed');
    }
    public function soldCount()
    {
        return $this->hasMany(Order::class, 'card_id')->where('status', 'completed');
    }

    public function ratedOrders()
    {
        return $this->hasMany(Order::class, 'card_id')->whereNotNull('rate');
    }

    public function reviews()
    {
        return $this->hasMany(Reviews::class, 'card_id');
    }

    public function getSoldCountAttribute()
    {
        return $this->sold->count();
    }

    public function getIsFavoriteAttribute()
    {
        $user = Auth::user() ?? Auth::guard("api")->user();
        if ($user) {
            // return $user;
            $fav_cards = FavoriteCard::where(['user_id' => $user->id, 'card_id' => $this->id])->first();
            if ($fav_cards) return 1;
        }
        return 0;
    }
}
