<?php

namespace App\Providers;

use App\Http\Controllers\Base;
use Illuminate\Cache\RateLimiting\Limit;
use Illuminate\Foundation\Support\Providers\RouteServiceProvider as ServiceProvider;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\RateLimiter;
use Illuminate\Support\Facades\Route;

class RouteServiceProvider extends ServiceProvider
{
    /**
     * The path to the "home" route for your application.
     *
     * Typically, users are redirected here after authentication.
     *
     * @var string
     */
    public const HOME = '/home';

    /**
     * Define your route model bindings, pattern filters, and other route configuration.
     *
     * @return void
     */
    public function boot()
    {
        $this->configureRateLimiting();

        $this->routes(function () {
            // Route::middleware('api')
            //     ->prefix('api')
            //     ->group(base_path('routes/api.php'));
            // Route::middleware('api')
            //     ->prefix('admin/v1/admin')
            //     ->group(base_path('routes/admin.php'));

            //? NOTE: here we separated API file according to user base. 

            Route::group(['middleware' => ['api']], function () {
                Route::group(['prefix' => 'api'], function(){
                    Route::group(['prefix' => 'v1'], function () {

                        // Route::group(['prefix' => '{locale}','middleware' => 'locale'], function (Request $locale) {
                            // else
                            Route::prefix('common')
                                ->group(base_path('routes/api.php'));
                            Route::prefix('admin')->middleware(['auth:api', 'role:sa|admin'])
                                ->group(base_path('routes/admin.php'));
                            Route::prefix('customer')->middleware(['auth:api', 'role:customer'])
                                ->group(base_path('routes/customer.php'));
                            Route::prefix('shop')->middleware(['auth:api', 'role:shop'])
                                ->group(base_path('routes/shop.php'));
                        // });
                        // Route::get('');
                        

                        Route::any('{path}', function () {
                            return Base::error('Url Not found, Check your URL first');
                        })->where('path', '.*');
                    });
                    Route::any('{path}', function () {
                        return Base::error('Url Not found, Check your URL first');
                    })->where('path', '.*');
                });
                
                
            });

            Route::middleware('web')
                ->group(base_path('routes/web.php'));
        });
    }

    /**
     * Configure the rate limiters for the application.
     *
     * @return void
     */
    protected function configureRateLimiting()
    {
        RateLimiter::for('api', function (Request $request) {
            return Limit::perMinute(60)->by($request->user()?->id ?: $request->ip());
        });
    }
}
