<?php

namespace App\Services\Repositories;

use App\Http\Controllers\Base;
use App\Models\AddressBook;
use App\Services\Interface\AddressBookInterface;
use App\Traits\GlobalTrait;
use Exception;
use Illuminate\Support\Facades\Auth;

class AddressBookRepository implements AddressBookInterface
{
    use GlobalTrait;

    public function index()
    {
        try {
            $data = $this->getUserAddresses();
            return Base::pass("All address list fetched", $data); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function add($request)
    {
        try {
            $user = Auth::user();
            $new_address =  new AddressBook();
            $new_address->user_id = $user->id;
            $new_address->name = $request->name;
            $new_address->phone = $request->phone;
            $new_address->address = $request->address;
            $new_address->post_code = $request->post_code;
            $new_address->type = $request->type;
            $new_address->city = $request->city;
            if (isset($request->note)) $new_address->note = $request->note;
            $new_address->save();
            $data = $this->getUserAddresses();
            return Base::pass("New address added successfully", $data); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }
    public function update($request)
    {
        try {
            $user = Auth::user();
            $addressBook = AddressBook::find($request->id);
            if (!$addressBook) return Base::fail("Address not found"); // DONE
            if ($addressBook->user_id != $user->id) return Base::fail("Unauthorized user"); // DONE


            if (isset($request->name)) $addressBook->name = $request->name;
            if (isset($request->phone)) $addressBook->phone = $request->phone;
            if (isset($request->address)) $addressBook->address = $request->address;
            if (isset($request->post_code)) $addressBook->post_code = $request->post_code;
            if (isset($request->city)) $addressBook->city = $request->city;
            if (isset($request->type)) $addressBook->type = $request->type;
            if (isset($request->note)) $addressBook->note = $request->note;

            $addressBook->save();
            $data = $this->getUserAddresses();
            return Base::pass("Address updated successfully", $data); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function delete($request)
    {
        try {
            $addressBook = AddressBook::find($request->id);
            if (!$addressBook) return Base::fail("Address Book not found"); // DONE
            $addressBook->delete();
            $data = $this->getUserAddresses();
            return Base::pass("Address book deleted successfully", $data); // DONE
        } catch (Exception $e) {
            Base::exception_fail($e);
        }
    }

    private function getUserAddresses()
    {
        $user = Auth::user();
        $addressBooks = $user->addressBooks;
        if (!$addressBooks) $addressBooks = [];
        return $addressBooks;
    }
}
