<?php

namespace App\Services\Repositories;

use App\Http\Controllers\Base;
use App\Models\FAQ;
use App\Services\Interface\FaqInterface;
use Exception;

class FaqRepository implements FaqInterface {

    public function index()
    {
        try{
            $faq = FAQ::all();
            return Base::pass("All faq list", $faq); // DONE
        }
        catch(Exception $e){
            return Base::exception_fail($e);
        }
    }

    public function add($request)
    {
        try{
            $faq = new FAQ();
            $faq->title = $request['title'];
            $faq->description = $request['description'];
            $faq->save();

            $data = $this->index();

            return Base::pass("New FAQ added successfully", $data->data); // DONE

        }
        catch(Exception $e){
            return Base::exception_fail($e);
        }
    }

    public function update($request)
    {
        try{
            $faq = FAQ::find($request['id']);

            if($faq == null) return Base::fail("Faq not found"); // DONE

            $faq->title = isset($request['title']) ? $request['title'] : $faq->title;
            $faq->description = isset($request['description']) ? $request['description'] : $faq->description;

            $faq->save();

            $data = $this->index();

            return Base::pass("FAQ updated successfully", $data->data); // DONE

        }
        catch(Exception $e){
            return Base::exception($e);
        }
    }

    public function delete($request)
    {
        try{
            $faq = FAQ::find($request['id']);

            if ($faq == null) return Base::fail("Faq not found"); // DONE

            $faq->delete();

            $data = $this->index();

            return Base::pass("FAQ deleted successfully", $data->data); // DONE

        }
        catch(Exception $e){
            return Base::exception($e);
        }
    }

    public function statusToggle($request)
    {
        try{
            //Code will be here
            $faq = FAQ::find($request['id']);
            if ($faq == null) return Base::fail("Faq not found");
            $faq->is_active = $request->is_active ? true : false;
            $faq->save();
            
            return Base::pass("FAQ Status has been updated"); // DONE
        }
        catch(Exception $e){
            return Base::exception($e);
        }
    }
}