<?php

namespace App\Services\Repositories;

use App\Http\Controllers\Base;
use App\Models\GeneralInfo;
use App\Services\Interface\GeneralInterface;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class GeneralRepository implements GeneralInterface
{

  public function getGeneralInfo(Request $request)
  {
    $validator = Validator::make($request->all(), [
      'type' => 'required|in:about-us,privacy-policy,terms-and-conditions,delete-message',
    ]);


    if ($validator->fails()) return Base::validation($validator);

    try {
      $lang = isset($request->lang) ? $request->lang : 'en';
      $info = GeneralInfo::where(['type' => $request->type, 'lang' => $lang])->first();
      if (!isset($info)) return Base::error("No information found!", null);

      return Base::success('Information of ' . $request->type, $info);
    } catch (Exception $e) {
      return Base::exception($e);
    }
  }

  public function updateGeneralInfo(Request $request)
  {
    $validator = Validator::make($request->all(), [
      'type' => 'required|in:about-us,privacy-policy,terms-and-conditions,delete-message',
      'info' => 'required',
    ]);

    if ($validator->fails()) return Base::validation($validator);

    try {
      $lang = isset($request->lang) ? $request->lang : 'en';
      $info = GeneralInfo::where(['type' => $request->type, 'lang' => $lang])->first();
      if (!isset($info)) {
        // return Base::error("No information found!", null);
        $info = new GeneralInfo();
        $info->type = $request->type;
      }

      $info->info = $request->info;
      $info->lang = $lang;
      $info->save();

      return Base::success('Information of ' . $request->type . ' updated successfully!', $info);
    } catch (Exception $e) {
      return Base::exception($e);
    }
  }

  // todo:: add other methods here
}
