<?php

namespace App\Services\Repositories;

use App\Http\Controllers\Base;
use App\Models\Notification;
use App\Services\Interface\NotificationInterface;
use Exception;
use Illuminate\Support\Facades\Validator;

class NotificationRepository implements NotificationInterface
{

    public function index($request)
    {
        try {
            $take = $request->take ?? 20;
            $user_id = auth()->user()->id;

            $data = [];

            $notifications = Notification::where('to_id', $user_id)->orderBy('id', 'DESC')
            ->with('from_user:id,name')
            ->with('to_user:id,name')
            // ->select('id', 'type', 'title_key as title', 'description_key as description', 'is_seen', 'created_at', 'value')
            ->get();

            $unseen_count = Notification::where('to_id', $user_id)->where('is_seen', false)->get()->count();

            if($notifications->count() > 0)
            {
                foreach ($notifications as $notification) {
                    if ($notification->type == 'order_processing' || 
                    $notification->type == 'order_completed' || 
                    $notification->type == 'order_cancelled' || 
                    $notification->type == 'order_cancelled_request' || 
                    $notification->type == 'order_rejected')
                    {
                        $data [] = [
                            'id' => $notification->id,
                            'title' => __('notification.'.$notification->title_key),
                            'type' => $notification->type,
                            'description' => __('notification.'.$notification->description_key, ['value' => $notification->value]),
                            'is_seen' => $notification->is_seen,
                            'created_at' => $notification->created_at,
                            'value' => $notification->value,
                        ];
                    }
                    if ($notification->type == 'new_order'){
                        $data [] = [
                            'id' => $notification->id,
                            'title' => __('notification.'.$notification->title_key),
                            'type' => $notification->type,
                            'description' => __('notification.'.$notification->description_key, ['name' => $notification->from_user->name]),
                            'is_seen' => $notification->is_seen,
                            'created_at' => $notification->created_at,
                            'value' => $notification->value,
                        ];
                    }

                    if($notification->type == 'referral'){
                        $data [] = [
                            'id' => $notification->id,
                            'title' => __('notification.'.$notification->title_key),
                            'type' => $notification->type,
                            'description' => __('notification.'.$notification->description_key, ['name' => $notification->from_user->name, 'value' => $notification->value]),
                            'is_seen' => $notification->is_seen,
                            'created_at' => $notification->created_at,
                            'value' => $notification->value,
                        ];
                    }

                    if($notification->type == 'purchase'){
                        $data [] = [
                            'id' => $notification->id,
                            'title' => __('notification.'.$notification->title_key),
                            'type' => $notification->type,
                            'description' => __('notification.'.$notification->description_key, ['value' => $notification->value]),
                            'is_seen' => $notification->is_seen,
                            'created_at' => $notification->created_at,
                            'value' => $notification->value,
                        ];
                    }
                }
            }


            $data = [
                'unseen_count' => $unseen_count,
                'notifications' => collect($data)->paginate($take),
            ];

            return Base::success("Notification List", $data);
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function show($request)
    {

        try {
            $validator = Validator::make($request->all(), [
                'id' => 'required|numeric',
            ]);

            if ($validator->fails())
                return Base::error($validator->errors()->first(), $validator->errors());

            $user_id = auth()->user()->id;

            $notification = Notification::where('id', $request->id)->first();
            if ($notification == null) return Base::error("Not Found", $notification); // DONE

            if ($notification->to_id != $user_id) return Base::error('Notification user does not match'); // DONE

            if ($notification->is_seen == 0) {
                $notification->is_seen = 1;
                $notification->save();
            }

            $item = [
                'id' => $notification->id,
                'title' => $notification->title_key,
                'type' => $notification->type,
                'description' => $notification->description_key,
                'is_seen' => $notification->is_seen,
                'created_at' => $notification->created_at,
            ];

            return Base::success('Notification fetched', $item); // DONE
        } catch (\Exception $e) {
            return Base::error('An error occurred!'); // DONE
        }
    }
}
