<?php

namespace App\Services\Repositories;

use App\Http\Controllers\Base;
use App\Models\ShopBoost;
use App\Models\User;
use App\Services\Interface\ShopBoostedInterface;
use Exception;

// use App\Services\Interface\ShopInterface;

class ShopBoostedRepository implements ShopBoostedInterface
{

    public function boostedShop($data)
    {
        try {
            //code...
            $res = ShopBoost::where('status', 'active')->with('shop')->take($data)->first();
            if (!$res) return Base::fail("List not found.");
            
            $data =  $res->shop;
            // $user = User::find($data->id);

            // if ($user->is_active == 0) $data = null;
            // else $data =  $res->shop;

            return Base::pass("Boosted Shop List", $data); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminBoostedShopIndex($data)
    {
        try {
            $take = isset($data['take']) ? $data['take'] : 20;
            $res = ShopBoost::with('shop')->paginate($take);
            return Base::pass("All boosted shop list", $res); // DONE
        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function adminBoostedShopFilter($request)
    {
        try {

            $search = isset($request['search']) ? $request['search'] : null;
            $take = isset($request['take']) ? $request['take'] : 20;

            $res = ShopBoost::with('shop')
                ->whereHas('shop', function ($q) use ($search) {
                    return $q->where('name', 'like', '%' . $search . '%');
                })
                // ->where('status','like', '%' . $search . '%')
                ->paginate($take);

            return Base::pass("All boosted shop list", $res); // DONE

        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function addBoostedShop($request)
    {
        try {
            $shop_id = $request['shop_id'];
            $start_date = Base::carbonParse($request['start_date']);
            $end_date = Base::carbonParse($request['end_date']);

            $shop = User::where(['id' => $shop_id, 'role' => 'shop'])->first();

            if (!$shop) return Base::fail("Shop not found"); // DONE

            $boosted_shop = ShopBoost::where('shop_id', $shop_id)->first();

            if ($boosted_shop) return Base::fail("Shop is enlisted in the list already"); // DONE

            $boosted_shop  = new ShopBoost();
            $boosted_shop->shop_id = $shop_id;
            $boosted_shop->start_date = $start_date;
            $boosted_shop->end_date = $end_date;
            $boosted_shop->status = 'init';

            $boosted_shop->save();

            $take = [
                "take" => 20
            ];

            $res = $this->adminBoostedShopIndex($take);

            $data = $res->data;

            return Base::pass("Shop is added in boosted shop list", $data); // DONE

        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function editBoostedShop($request)
    {
        try {
            $boosted_id = $request['boosted_id'];

            $boosted_shop = ShopBoost::find($boosted_id);
            if (!$boosted_shop) return Base::fail("Data is not found"); // DONE
            // $shop_id = isset($request['shop_id']) ? $request['shop_id'] : $boosted_shop->shop_id;
            $start_date = isset($request['start_date']) ? Base::carbonParse($request['start_date']) : $boosted_shop->start_date;
            $end_date = isset($request['end_date']) ? Base::carbonParse($request['end_date']) : $boosted_shop->end_date;

            $status = isset($request['status']) ? $request['status'] : $boosted_shop->status;

            // $boosted_shop->shop_id = $shop_id;
            $boosted_shop->start_date = $start_date;
            $boosted_shop->end_date = $end_date;
            $boosted_shop->status = $status;

            $boosted_shop->save();

            $take = [
                "take" => 20
            ];

            $res = $this->adminBoostedShopIndex($take);

            $data = $res->data;

            return Base::pass("Shop is updated in boosted shop list", $data); // DONE

        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }

    public function deleteBoostedShop($request)
    {
        try {
            $boosted_id = $request['boosted_id'];

            $boosted_shop = ShopBoost::find($boosted_id);
            if (!$boosted_shop) return Base::fail("Data is not found"); // DONE

            $boosted_shop->delete();

            $take = [
                "take" => 20
            ];

            $res = $this->adminBoostedShopIndex($take);

            $data = $res->data;

            return Base::pass("Shop is deleted from boosted shop list", $data); // DONE

        } catch (Exception $e) {
            return Base::exception_fail($e);
        }
    }
}
