<?php

use App\Http\Controllers\Admin\CategoryController;
use App\Http\Controllers\Admin\VoucherController;
use App\Http\Controllers\AuthController;
use App\Http\Controllers\Base;
use App\Http\Controllers\ContactUsController;
use App\Http\Controllers\HomeController;
use App\Http\Controllers\NotificationController;
use App\Http\Controllers\OrderController;
use App\Http\Controllers\PostCardController;
use App\Http\Controllers\ProfileController;
use App\Http\Controllers\ShopController;
use App\Http\Controllers\GeneralController;
use App\Http\Controllers\TestController;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Support\Facades\Route;


//! check RouteServerProvider.php for more info

Route::group([], function () {
    /*
    |--------------------------------------------------------------------------
    | Auth
    |--------------------------------------------------------------------------*/
    Route::group(['prefix' => 'auth',], function () {
        Route::get('/user', [AuthController::class, 'getAuthUser'])->middleware('auth:api');
        Route::post('/login', [AuthController::class, 'login'])->middleware('throttle:5,1')->name('login');
        Route::post('/register', [AuthController::class, 'register']);
        Route::post('/verify', [AuthController::class, 'verifyOTP'])->middleware('auth:api', 'throttle:5,1')->name('verify.otp');
        Route::post('/resend-otp', [AuthController::class, 'resendOTP']);
        Route::post('/forget-password', [AuthController::class, 'resendOTP']);
        Route::post('/forget-password-otp-verify', [AuthController::class, 'verifyOTP']);
        Route::post('/reset-password', [AuthController::class, 'resetPassword']);
        Route::post('/logout', [AuthController::class, 'logout'])->middleware('auth:api');
        Route::post('social-login', [AuthController::class, 'socialLogin']);
        


        Route::group(['prefix' => 'profile', 'middleware' => ['auth:api', 'role:customer|shop|sa|admin']], function () {
            Route::get('/index', [ProfileController::class, 'index']);
            Route::post('/update', [ProfileController::class, 'update']);
            Route::post('/change-password', [ProfileController::class, 'changePassword'])->middleware('throttle:5,1');
            Route::post('/change-language', [ProfileController::class, 'changeLanguage']);
            Route::post('/toggle-settings', [ProfileController::class, 'togglePushOrEmailNotification']);
            Route::post('/delete', [ProfileController::class, 'delete']);
            Route::post('user-verification-id-upload', [AuthController::class, 'vIDVerify']);
            Route::get('user-verification-info', [ProfileController::class, 'vIDVerifyInfo']);
            Route::get('/get-refer-code', [ProfileController::class, 'getReferCode']);

            Route::get('get-delete-view', [ProfileController::class, 'deleteText']);
            Route::get('get-point-details', [ProfileController::class, 'pointDetails']);
        });
    });

    Route::group([], function () {
        Route::get('/home', [HomeController::class, 'index']);
        Route::get('get-category', [HomeController::class, 'getCategory']);
        Route::get('get-recommended-category', [CategoryController::class, 'getRecommendedCategoryList']);

        //* get all cards and filter.
        // Route::get('post-cards', [PostCardController::class, 'index']);   
    });

    //* Notification 

    Route::group(['prefix' => 'notification', 'middleware' => ['auth:api']], function () {
        Route::get('index', [NotificationController::class, 'index']);
        Route::post('show', [NotificationController::class, 'show']);
        Route::post('mark-all-read', [NotificationController::class, 'markAllRead']);
        Route::post('mark-all-unread', [NotificationController::class, 'markAllUnread']);
    });


    Route::group(['prefix' => 'post-card'], function () {

        Route::any('index', [PostCardController::class, 'index'])->name('common');
        Route::any('show', [PostCardController::class, 'show']);

        Route::get('show-reviews', [PostCardController::class, 'showReviews']);

        Route::any('get-recommended-cards', [PostCardController::class, 'index'])->name('get.recommended.card');
        Route::any('get-popular-cards', [PostCardController::class, 'index'])->name('get.popular.card');
        Route::any('get-new-cards', [PostCardController::class, 'index'])->name('get.new.card');

        // // ========================
        // // Post Card
        // // ========================
        // Route::group(['prefix' => 'post-card'], function () {
        //     Route::any('/index', [PostCardController::class, 'index']);
        //     Route::get('/show', [PostCardController::class, 'show']);
        // Route::post('/add', [PostCardController::class, 'add']);
        //     Route::post('/update', [PostCardController::class, 'update']);
        //     Route::post('/delete', [PostCardController::class, 'delete']);
        // });
    });
    Route::group(['prefix' => 'shop'], function () {
        Route::any('index', [ShopController::class, 'index']);
        Route::any('get-cards', [PostCardController::class, 'index'])->name('shop_cards');
    });

    // ========================
    // Contact Us
    // ========================
    Route::group(['prefix' => 'contact-us'], function () {
        Route::post('/add', [ContactUsController::class, 'add']);
    });

    Route::post('set-device-token', [NotificationController::class, 'setDeviceToken'])->middleware('auth:api');
    Route::post('new-notification', [NotificationController::class, 'setNotification'])->middleware('auth:api');



    Route::post('check-voucher', [VoucherController::class, 'checkVoucher'])->middleware('auth:api');

    // General Info(about us, privacy-policy, terms and conditions)
    Route::get('general-info', [GeneralController::class, 'getGeneralInfo']);
    Route::group(['middleware' => ['auth:api', 'role:customer|shop|sa|admin']], function () {
        Route::post('general-info/update', [GeneralController::class, 'updateGeneralInfo']);
    });

    Route::get('referral-user', [AuthController::class, 'referralUser']);




    //! TEST Function route, don't need to PR that

    Route::get('get-all-payment-intent', [OrderController::class, 'allPaymentIntent']);
    Route::post('/social-login-test', [TestController::class, 'socialTest']);
    Route::post('/stripe-payment-intent', [OrderController::class, 'createPaymentIntent']);


    Route::get('test', function () {

        $amount = 84;
        $tax = 10;

        return Base::taxCalculation($amount, $tax);

        $country_code = null;
        $code = 'z3h';
        switch ($code) {
            case 'en':
                $country_code = 'US';
                break;
            case 'bn':
                $country_code = 'BD';
                break;
            case 'ja':
                $country_code = 'JP';
                break;
            case 'zh':
                $country_code = 'CN';
                break;
        }
        return $country_code;
        // $users = User::all();
        // foreach($users as $user){
        //     if($user->lang_code == 'ja'){
        //         $user->country_code = 'JP';
        //         $user->save();
        //     }
        //     if($user->lang_code == 'en'){
        //         $user->country_code = 'EN';
        //         $user->save();
        //     }
        // }
        return Carbon::now()->locale('ja')->isoFormat('LLLL');
    });
});
